==> RoughlyEnoughItems-fabric-18.0.804-sources.jar: me/shedaniel/rei/impl/client/gui/config/components/ConfigOptionWidget.java <==
/*
 * This file is licensed under the MIT License, part of Roughly Enough Items.
 * Copyright (c) 2018, 2019, 2020, 2021, 2022, 2023 shedaniel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package me.shedaniel.rei.impl.client.gui.config.components;

import me.shedaniel.clothconfig2.api.animator.NumberAnimator;
import me.shedaniel.clothconfig2.api.animator.ValueAnimator;
import me.shedaniel.math.Point;
import me.shedaniel.math.Rectangle;
import me.shedaniel.rei.api.client.gui.widgets.Label;
import me.shedaniel.rei.api.client.gui.widgets.Widget;
import me.shedaniel.rei.api.client.gui.widgets.WidgetWithBounds;
import me.shedaniel.rei.api.client.gui.widgets.Widgets;
import me.shedaniel.rei.api.client.util.MatrixUtils;
import me.shedaniel.rei.impl.client.gui.config.ConfigAccess;
import me.shedaniel.rei.impl.client.gui.config.options.AllREIConfigOptions;
import me.shedaniel.rei.impl.client.gui.config.options.CompositeOption;
import me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils;
import me.shedaniel.rei.impl.client.gui.text.TextTransformations;
import net.minecraft.class_156;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_364;
import net.minecraft.class_5250;
import net.minecraft.class_5481;
import org.jetbrains.annotations.Nullable;
import org.joml.Matrix4f;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.literal;
import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.translatable;

public class ConfigOptionWidget {
    public static <T> WidgetWithBounds create(ConfigAccess access, CompositeOption<T> option, int width) {
        List<Widget> widgets = new ArrayList<>();
        int[] stableHeight = {12};
        int[] height = {12};
        Label fieldNameLabel;
        widgets.add(fieldNameLabel = Widgets.createLabel(new Point(0, 0), TextTransformations.highlightText(option.getName().method_27661(), option.getOptionNameHighlight(), style -> style.method_36139(0xFFC0C0C0)))
                .leftAligned()
                .rainbow(Objects.equals(option.getId(), AllREIConfigOptions.RAINBOW.getId())));
        WidgetWithBounds optionValue = ConfigOptionValueWidget.create(access, option, width - 10 - fieldNameLabel.getBounds().width);
        widgets.add(Widgets.withTranslate(optionValue, () -> new Matrix4f().translate(width - optionValue.getBounds().width - optionValue.getBounds().x, 0, 0)));
        widgets.add(new WidgetWithBounds() {
            final class_5250 description = class_156.method_656(() -> {
                class_5250 description = option.getDescription().method_27661();
                if (description.getString().endsWith(".desc")) {
                    return literal("");
                } else {
                    return TextTransformations.highlightText(description, option.getOptionDescriptionHighlight(), style -> style.method_36139(0xFF757575));
                }
            });
            
            final List<class_5481> split = class_310.method_1551().field_1772.method_1728(description, width);
            final boolean hasPreview = option.hasPreview();
            final Label previewLabel = Widgets.createLabel(new Point(), translatable("config.rei.texts.preview"))
                    .color(0xFFA5F4FF)
                    .hoveredColor(0xFFD1FAFF)
                    .noShadow()
                    .clickable()
                    .onClick($ -> clickPreview())
                    .rightAligned();
            @Nullable
            WidgetWithBounds preview = null;
            boolean previewVisible = false;
            Matrix4f previewTranslation = new Matrix4f();
            final NumberAnimator<Float> previewHeight = ValueAnimator.ofFloat()
                    .withConvention(() -> previewVisible ? preview.getBounds().getHeight() : 0f, ValueAnimator.typicalTransitionTime());
            boolean nextLinePreview = false;
            
            {
                stableHeight[0] += 12 * split.size();
                if (hasPreview) {
                    int lastWidth = class_310.method_1551().field_1772.method_30880(split.get(split.size() - 1));
                    if (lastWidth + this.previewLabel.getBounds().width + 10 > width) {
                        nextLinePreview = true;
                        stableHeight[0] += 12;
                    }
                }
            }
            
            @Override
            public Rectangle getBounds() {
                return new Rectangle(0, 12, width, 12 + 12 * split.size());
            }
            
            @Override
            public void method_25394(class_332 graphics, int mouseX, int mouseY, float delta) {
                this.previewHeight.update(delta);
                if (ConfigUtils.isReducedMotion()) this.previewHeight.completeImmediately();
                height[0] = stableHeight[0] + Math.round(this.previewHeight.value());
                
                for (int i = 0; i < split.size(); i++) {
                    graphics.method_51430(class_310.method_1551().field_1772, split.get(i), 0, 12 + 12 * i, -1, false);
                }
                
                if (hasPreview) {
                    if (nextLinePreview) {
                        this.previewLabel.setPoint(new Point(width, 12 + 12 * split.size()));
                    } else {
                        this.previewLabel.setPoint(new Point(width, 12 + 12 * split.size() - 12));
                    }
                    
                    this.previewLabel.method_25394(graphics, mouseX, mouseY, delta);
                    
                    if (this.preview != null && this.previewHeight.value() > 0.1f) {
                        Rectangle rectangle = MatrixUtils.transform(graphics.method_51448().method_23760().method_23761(), new Rectangle(0, 24 + 12 * split.size() - (nextLinePreview ? 0 : 12), width, this.previewHeight.value()));
                        graphics.method_44379(rectangle.x, rectangle.y, rectangle.getMaxX(), rectangle.getMaxY());
                        this.previewTranslation = new Matrix4f().translate(0, 12 + 12 * split.size(), 100);
                        this.preview.method_25394(graphics, mouseX, mouseY, delta);
                        graphics.method_44380();
                    }
                }
            }
            
            private void clickPreview() {
                if (this.preview == null) {
                    this.preview = option.getPreviewer().preview(width, () -> access.get(option));
                    this.preview = Widgets.withTranslate(this.preview, () -> this.previewTranslation);
                }
                
                this.previewVisible = !this.previewVisible;
            }
            
            @Override
            public List<? extends class_364> method_25396() {
                if (this.preview != null && this.previewHeight.value() > 0.1f) return List.of(this.previewLabel, this.preview);
                return List.of(this.previewLabel);
            }
        });
        
        height[0] = stableHeight[0];
        return Widgets.concatWithBounds(() -> new Rectangle(0, 0, width, height[0]), widgets);
    }
}

==> RoughlyEnoughItems-fabric-19.0.806-sources.jar: me/shedaniel/rei/impl/client/gui/config/components/ConfigOptionWidget.java <==
/*
 * This file is licensed under the MIT License, part of Roughly Enough Items.
 * Copyright (c) 2018, 2019, 2020, 2021, 2022, 2023 shedaniel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package me.shedaniel.rei.impl.client.gui.config.components;

import me.shedaniel.clothconfig2.api.animator.NumberAnimator;
import me.shedaniel.clothconfig2.api.animator.ValueAnimator;
import me.shedaniel.math.Point;
import me.shedaniel.math.Rectangle;
import me.shedaniel.rei.api.client.gui.widgets.Label;
import me.shedaniel.rei.api.client.gui.widgets.Widget;
import me.shedaniel.rei.api.client.gui.widgets.WidgetWithBounds;
import me.shedaniel.rei.api.client.gui.widgets.Widgets;
import me.shedaniel.rei.impl.client.gui.config.ConfigAccess;
import me.shedaniel.rei.impl.client.gui.config.options.AllREIConfigOptions;
import me.shedaniel.rei.impl.client.gui.config.options.CompositeOption;
import me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils;
import me.shedaniel.rei.impl.client.gui.text.TextTransformations;
import net.minecraft.class_156;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_364;
import net.minecraft.class_5250;
import net.minecraft.class_5481;
import org.jetbrains.annotations.Nullable;
import org.joml.Matrix4f;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.literal;
import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.translatable;

public class ConfigOptionWidget {
    public static <T> WidgetWithBounds create(ConfigAccess access, CompositeOption<T> option, int width) {
        List<Widget> widgets = new ArrayList<>();
        int[] stableHeight = {12};
        int[] height = {12};
        Label fieldNameLabel;
        widgets.add(fieldNameLabel = Widgets.createLabel(new Point(0, 0), TextTransformations.highlightText(option.getName().method_27661(), option.getOptionNameHighlight(), style -> style.method_36139(0xFFC0C0C0)))
                .leftAligned()
                .rainbow(Objects.equals(option.getId(), AllREIConfigOptions.RAINBOW.getId())));
        WidgetWithBounds optionValue = ConfigOptionValueWidget.create(access, option, width - 10 - fieldNameLabel.getBounds().width);
        widgets.add(Widgets.withTranslate(optionValue, () -> new Matrix4f().translate(width - optionValue.getBounds().width - optionValue.getBounds().x, 0, 0)));
        widgets.add(new WidgetWithBounds() {
            final class_5250 description = class_156.method_656(() -> {
                class_5250 description = option.getDescription().method_27661();
                if (description.getString().endsWith(".desc")) {
                    return literal("");
                } else {
                    return TextTransformations.highlightText(description, option.getOptionDescriptionHighlight(), style -> style.method_36139(0xFF757575));
                }
            });
            
            final List<class_5481> split = class_310.method_1551().field_1772.method_1728(description, width);
            final boolean hasPreview = option.hasPreview();
            final Label previewLabel = Widgets.createLabel(new Point(), translatable("config.rei.texts.preview"))
                    .color(0xFFA5F4FF)
                    .hoveredColor(0xFFD1FAFF)
                    .noShadow()
                    .clickable()
                    .onClick($ -> clickPreview())
                    .rightAligned();
            @Nullable
            WidgetWithBounds preview = null;
            boolean previewVisible = false;
            Matrix4f previewTranslation = new Matrix4f();
            final NumberAnimator<Float> previewHeight = ValueAnimator.ofFloat()
                    .withConvention(() -> previewVisible ? preview.getBounds().getHeight() : 0f, ValueAnimator.typicalTransitionTime());
            boolean nextLinePreview = false;
            
            {
                stableHeight[0] += 12 * split.size();
                if (hasPreview) {
                    int lastWidth = class_310.method_1551().field_1772.method_30880(split.get(split.size() - 1));
                    if (lastWidth + this.previewLabel.getBounds().width + 10 > width) {
                        nextLinePreview = true;
                        stableHeight[0] += 12;
                    }
                }
            }
            
            @Override
            public Rectangle getBounds() {
                return new Rectangle(0, 12, width, 12 + 12 * split.size());
            }
            
            @Override
            public void method_25394(class_332 graphics, int mouseX, int mouseY, float delta) {
                this.previewHeight.update(delta);
                if (ConfigUtils.isReducedMotion()) this.previewHeight.completeImmediately();
                height[0] = stableHeight[0] + Math.round(this.previewHeight.value());
                
                for (int i = 0; i < split.size(); i++) {
                    graphics.method_51430(class_310.method_1551().field_1772, split.get(i), 0, 12 + 12 * i, -1, false);
                }
                
                if (hasPreview) {
                    if (nextLinePreview) {
                        this.previewLabel.setPoint(new Point(width, 12 + 12 * split.size()));
                    } else {
                        this.previewLabel.setPoint(new Point(width, 12 + 12 * split.size() - 12));
                    }
                    
                    this.previewLabel.method_25394(graphics, mouseX, mouseY, delta);
                    
                    if (this.preview != null && this.previewHeight.value() > 0.1f) {
                        Rectangle rectangle = new Rectangle(0, 24 + 12 * split.size() - (nextLinePreview ? 0 : 12), width, this.previewHeight.value());
                        graphics.method_44379(rectangle.x, rectangle.y, rectangle.getMaxX(), rectangle.getMaxY());
                        this.previewTranslation = new Matrix4f().translate(0, 12 + 12 * split.size(), 100);
                        this.preview.method_25394(graphics, mouseX, mouseY, delta);
                        graphics.method_44380();
                    }
                }
            }
            
            private void clickPreview() {
                if (this.preview == null) {
                    this.preview = option.getPreviewer().preview(width, () -> access.get(option));
                    this.preview = Widgets.withTranslate(this.preview, () -> this.previewTranslation);
                }
                
                this.previewVisible = !this.previewVisible;
            }
            
            @Override
            public List<? extends class_364> method_25396() {
                if (this.preview != null && this.previewHeight.value() > 0.1f) return List.of(this.previewLabel, this.preview);
                return List.of(this.previewLabel);
            }
        });
        
        height[0] = stableHeight[0];
        return Widgets.concatWithBounds(() -> new Rectangle(0, 0, width, height[0]), widgets);
    }
}
